package eu.mhsl.craftattack.spawn.appliances.displayName;

import eu.mhsl.craftattack.spawn.Main;
import eu.mhsl.craftattack.spawn.appliance.Appliance;
import eu.mhsl.craftattack.spawn.appliances.adminMarker.AdminMarker;
import eu.mhsl.craftattack.spawn.appliances.adminMarker.AdminMarkerListener;
import eu.mhsl.craftattack.spawn.appliances.afkTag.AfkTag;
import eu.mhsl.craftattack.spawn.appliances.outlawed.Outlawed;
import eu.mhsl.craftattack.spawn.appliances.sleepTag.SleepTag;
import eu.mhsl.craftattack.spawn.appliances.yearRank.YearRank;
import eu.mhsl.craftattack.spawn.util.server.Floodgate;
import eu.mhsl.craftattack.spawn.util.text.ComponentUtil;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.ComponentBuilder;
import net.kyori.adventure.text.TextComponent;
import net.kyori.adventure.text.event.HoverEvent;
import net.kyori.adventure.text.format.NamedTextColor;
import net.kyori.adventure.text.format.TextColor;
import org.bukkit.entity.Player;
import org.bukkit.event.Listener;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.List;
import java.util.function.Supplier;
import java.util.logging.Level;

public class DisplayName extends Appliance {
    public interface Prefixed {
        @Nullable
        Component getNamePrefix(Player player);
    }

    public void update(Player player) {
        TextColor playerColor = this.queryAppliance(AdminMarker.class).getPlayerColor(player);
        List<Supplier<Component>> prefixes = List.of(
            () -> this.queryAppliance(Outlawed.class).getNamePrefix(player),
            () -> this.queryAppliance(YearRank.class).getNamePrefix(player),
            () -> this.queryAppliance(AfkTag.class).getNamePrefix(player),
            () -> this.queryAppliance(SleepTag.class).getNamePrefix(player)
        );

        ComponentBuilder<TextComponent, TextComponent.Builder> playerName = Component.text();
        prefixes.forEach(supplier -> {
            Component prefix = supplier.get();
            if(prefix == null) return;
            playerName
                .append(prefix)
                .append(ComponentUtil.clearedSpace());
        });

        if(Floodgate.isBedrock(player)) {
            playerName
                .append(
                    Component.text("\uD83C\uDFAE", NamedTextColor.GRAY)
                        .hoverEvent(HoverEvent.showText(Component.text(
                            String.format("%s spielt die Minecraft: Bedrock Edition", player.getName())
                        )))
                )
                .append(ComponentUtil.clearedSpace());
        }

        playerName.append(Component.text(player.getName(), playerColor));

        this.setGlobal(player, playerName.build());
    }

    private void setGlobal(Player player, Component component) {
        try {
            player.customName(component);
            player.displayName(component);
            player.playerListName(component);
        } catch(Exception e) {
            Main.instance().getLogger().log(Level.SEVERE, e, e::getMessage);
        }
    }

    @Override
    @NotNull
    protected List<Listener> listeners() {
        return List.of(new AdminMarkerListener());
    }
}
